import sys
import time
import argparse
import subprocess
from collections import deque
from .graph_statistic import graph
from .sound import Sound as play_sound
from .statistics_db import (
    save_today_entry,
    read_all_entry,
    read_today_entry,
    delete_today_entry,
    delete_all_entry,
    close_db,
)

try:
    assert sys.version_info >= (3, 10, 13)
except Exception:
    raise SystemExit("Pomodoro Timer works with Python 3.10.13 or higher.")


class PomodoroTimer:
    """
    Created by Alexey Patsukov (patsuckow@yandex.ru) 🇷🇺
    CLI Pomodoro Timer - https://github.com/patsuckow/pomodorotimer
    """
    def __init__(self) -> None:
        self.mess = "Input min: 25/5/15/30 for your Pomodoro, or 0 for exit: "
        self.mess_two = "\nPomodoro's timer work has been interrupted."
        self.mess_three = "One Pomodoro cycle is over."
        self.scale_progress = ""
        self.clear_up_str = f"\x1b[A{79*' '}\r"
        self.stack_watch = deque(
            list("🕛🕧🕐🕜🕑🕝🕒🕞🕓🕟🕔🕠🕕🕡🕖🕢🕗🕣🕘🕤🕙🕥🕚🕦")
        )
        self.process_argument()
        self.pomodoro_setup()

    def process_argument(self) -> None:
        """
        Check if the correct one for working with statistics is passed;
        And depending on this, we make requests to the database or not.
        """
        argument = self.arg()
        if argument in ["today", "all", "delete-today", "delete-all"]:
            if argument == "today":
                dates, work, relaxation = read_today_entry()
                graph(dates, work, relaxation)
            elif argument == "all":
                dates, work, relaxation = read_all_entry()
                graph(dates, work, relaxation)
            elif argument == "delete-today":
                delete_today_entry()
            elif argument == "delete-all":
                delete_all_entry()

            close_db()
            self.stop_pomodoro()

    def pomodoro_setup(self) -> None:
        self.clear_console()

        try:
            while True:
                try:
                    self.show_console_cursor()
                    self.min = int(input(self.mess))
                except ValueError:
                    self.write_flush(self.clear_up_str)
                    continue

                self.write_flush(self.clear_up_str)

                if self.min in [5, 15, 25, 30]:
                    self.run_pomodoro()
                elif self.min == 0:
                    self.write_flush(self.clear_up_str)
                    if self.scale_progress != "":
                        print(self.scale_progress + self.mess_two)
                    self.stop_pomodoro()
        except KeyboardInterrupt:
            # interrupt processing (Ctrl + C)
            print(self.mess_two)
            self.stop_pomodoro()

    @staticmethod
    def clear_console() -> None:
        if sys.platform in ("linux", "darwin"):
            subprocess.call("clear", shell=True)
        elif sys.platform in ("win32", "cygwin"):
            subprocess.call("cls", shell=True)

    @staticmethod
    def arg() -> str:
        """Get the value of the argument --statistic passed to the script"""
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "--statistic",
            type=str,
            default="",
            help="provide a string: 'today', 'all', 'delete-today', "
            "'delete-all' (default: '')",
        )

        return parser.parse_args().statistic

    def progress_scale(self) -> str:
        """
        Colorize the timer scale for pomodoro in a different color, depending on
        the set time.
        """
        match self.min:
            case 5:
                self.scale_progress += f"\x1b[42m\x1b[1m {self.min} \x1b[0m"
            case 15:
                self.scale_progress += f"\x1b[42m\x1b[1m  {self.min}  \x1b[0m"
            case 25:
                self.scale_progress += f"\x1b[41m\x1b[1m   {self.min}   \x1b[0m"
            case 30:
                self.scale_progress += f"\x1b[42m\x1b[1m    {self.min}    \x1b[0m"

        self.write_flush(self.clear_up_str)

        return self.scale_progress

    def run_reverse_timer(self) -> None:
        """Countdown"""
        self.write_flush("\n" + self.clear_up_str)
        
        for i in range(self.min * 60, 0, -1):
            self.write_flush(f"\r{self.animated_clock()}{self.lead_time(i)} ")
            # The code execution delay (by 1 second) allows you to change the display
            #  of a string with a timer every second, t.e. this is the timer itself.
            # (For yourself: If you change the code, then for quick checking of 
            # performance, comment on this line and the code will be executed instantly)
            time.sleep(1)

        self.write_flush(f"\r{35*' '}\r")

    @staticmethod
    def write_flush(string: str) -> None:
        """
        The sys.stdout.flush() method is needed to clear the buffer output of
        the standard output stream. T.e. all data that has been delayed in the
        output buffer will be sent to the output device (for example, to the console)
        immediately. This is useful when you need to make sure that all the data was
        printed before the code continues to execute.
        """
        sys.stdout.write(string)
        sys.stdout.flush()

    @staticmethod
    def hide_console_cursor() -> None:
        """Hide console cursor: '\x1b[?25l'
        https://en.wikipedia.org/wiki/ANSI_escape_code#CSI_codes
        ('\033' same as '\x1b')
        """
        sys.stdout.write("\x1b[?25l")

    @staticmethod
    def show_console_cursor() -> None:
        """Show console cursor: '\x1b[?25h'"""
        sys.stdout.write("\x1b[?25h")

    def lead_time(self, sec: int) -> str:
        """Elapsed Time Calculation"""
        minutes = int(sec // 60)
        seconds = round(sec % 60, 3)
        if seconds // 10 == 0:
            seconds = "0" + str(seconds)
        if self.min == 25:
            return f" Pomodoro timer: \x1b[31m\x1b[1m{minutes}:{seconds}\x1b[0m"
        elif self.min in [5, 15, 30]:
            return f" Pomodoro timer: \x1b[32m\x1b[1m{minutes}:{seconds}\x1b[0m"

    def animated_clock(self) -> str:
        """
        To extract the first element from the list and place it at the end of
        the list, for greater performance, we will use the stack instead of
        the standard list methods .append() and .pop().
        https://docs.python.org/3/library/collections.html?#collections.deque
        """
        clock = self.stack_watch[0]
        self.stack_watch.appendleft(self.stack_watch.pop())

        return clock

    def save_statistics(self) -> None:
        """INSERT / UPDATE log statistic in DB

        According to the principle of the Pomodoro technique - if you are
        distracted while doing work, then you must interrupt the current time
        and start a new Pomodoro. And this means that you need to write time
        in to the database only after the specified time has passed and not
        earlier.

        If you follow this principle, the Pomodoro Technique will be for you
        exactly the tool for which it was invented, namely, not to calculate
        the time of your work and breaks in work, but to make you concentrate
        on continuous and concentrated execution of works during one Pomodoro,
        i.e. within 25 minutes.
        """
        work = 0
        relaxation = 0
        if self.min in [5, 15, 30]:
            relaxation = self.min
        else:
            work = self.min

        save_today_entry(work, relaxation)

    def show_pop_up_notice(self):
        """
        Show pop-up notification on Linux/macOS desktop.
        (Windows OS does not support the display of pop-up notifications.)
        """
        title = "⏰ Pomodoro: "
        info = f"{self.min} min TIME'S UP!"

        if sys.platform.startswith("linux"):
            command = ["notify-send", title, info]
        elif sys.platform == "darwin":  # macOS
            command = [
                "osascript",
                "-e",
                f'display notification "{info}" with title "{title}"',
            ]
        else:
            return  # Do nothing if not on supported platform

        subprocess.run(command, check=False)

    def check_one_pomodoro_cycle(self) -> None:
        """
        Checking the end of the pomodoro cycle
        """
        if self.min in [15, 30]:
            return print(self.mess_three), self.stop_pomodoro()

    def run_pomodoro(self) -> None:
        """Run pomodoro cicle"""
        # Нide the cursor display from the console
        self.hide_console_cursor()
        # Start the countdown timer
        self.run_reverse_timer()
        # Display (update display) the pomodoro progress bar
        print(self.progress_scale())
        # At the end of the timer, play the sound signal
        self.show_pop_up_notice()
        # Save statistics in the database
        play_sound()
        # Show pop-up notice notification
        self.save_statistics()
        # Check if the latter introduced 15 or 30, then we complete 
        # the pomodoro execution cycle
        self.check_one_pomodoro_cycle()
    
    def stop_pomodoro(self) -> None:
        self.show_console_cursor()
        exit(0)


import os
import sounddevice as sd
import numpy as np
from scipy.io import wavfile

class Sound:
    def __init__(self):
        self.path = os.path.abspath(os.path.join(os.path.dirname(__file__), "tic-tic.wav"))
        if os.path.exists(self.path):
            self.play_wav_sound()
        else:
            self.play_default_sound()

    @staticmethod
    def play_default_sound() -> None:
        """
        Sound generation with a frequency of 432 Hz and a duration of 864 milliseconds
        """
        frequency = 432
        duration = 0.864
        # Timeline (44100 Hz - sampling rate)
        t = np.linspace(0, duration, int(44100 * duration), endpoint=False)
        # Generation of sinusoidal sound
        data = 0.5 * np.sin(2 * np.pi * frequency * t)
        # Playback
        sd.play(data, 44100)
        # Waiting for the playback to end
        sd.wait()
    
    def play_wav_sound(self) -> None:
        """Play wav file"""
        # Reading audio file
        fs, data = wavfile.read(self.path)
        # Playback
        sd.play(data, fs)
        # Waiting for the playback to end
        sd.wait()

if __name__ == "__main__":
    Sound()


from .pomodorotimer import PomodoroTimer
from .graph_statistic import graph
from .sound import Sound as play_sound
from .statistics_db import (
    save_today_entry,
    read_all_entry,
    read_today_entry,
    delete_today_entry,
    delete_all_entry,
    close_db,
)



import os
import datetime
import calendar
import webbrowser
import matplotlib.pyplot as plt


class PeriodPomodoros:
    def __init__(
        self, dates: list, work: list, relaxation: list, name_svg: str
    ) -> None:
        # the width of the bars and size of the figure img
        if len(work) > 2:
            width = 0.4
            fig_size = (9.2, 7)
        else:
            width = 0.2
            fig_size = (7, 6)

        name_periods = []
        for i in dates:
            year, month, day = i.split("-")
            date = datetime.datetime(int(year), int(month), int(day)).weekday()
            name_periods.append(calendar.day_name[date] + "\n")

        if len(work) > 21:
            days = dates
        else:
            days = [k + j for k, j in zip(name_periods, dates)]

        x = range(len(days))  # the label locations
        fig, ax = plt.subplots(figsize=fig_size)
        rects_work = ax.bar([i - width / 2 for i in x], work, width, label="work")
        rects_relaxation = ax.bar([i + width / 2 for i in x], relaxation, width, label="relaxation")
        ax.set_title(f"Statistics Pomodoro`s for the last {len(work)} days:")
        ax.set_xticks(x)
        ax.set_xticklabels(days)
        ax.legend(bbox_to_anchor=(0.56, -0.2))
        ax.set_ylabel("Minutes")

        if len(work) > 7:
            for label in ax.xaxis.get_ticklabels():
                label.set_rotation(45 if len(work) <= 14 else 90)

        self.auto_label(rects_work, ax)
        self.auto_label(rects_relaxation, ax)
        fig.tight_layout()
        plt.savefig(name_svg, bbox_inches="tight")

    @staticmethod
    def auto_label(rects: list, ax) -> None:
        """
        Attach a text label above each bar in *rects*, displaying its height.
        """
        for rect in rects:
            height = rect.get_height()
            ax.annotate(
                "{}".format(height),
                xy=(rect.get_x() + rect.get_width() / 2, height),
                xytext=(0, 3),  # 3 points vertical offset
                textcoords="offset points",
                ha="center",
                va="bottom",
            )


class CountPomodorosTime:
    def __init__(self, work: list, relaxation: list, name_svg: str) -> None:
        work = sum(work)
        relaxation = sum(relaxation)

        fig, ax = plt.subplots(figsize=(9.2, 4.5))
        wedges, texts, autotexts = ax.pie(
            [work, relaxation],
            autopct=lambda pct: self.calc_pct(pct, [work, relaxation + 1]),
            textprops=dict(color="w"),
        )

        ax.legend(
            wedges,
            [
                "work: {:.2f} hours".format(work / 60),
                "relaxation: {:.2f} hours".format(relaxation / 60),
            ],
            title="Count Pomodoro`s time:\n",
            loc="center left",
            bbox_to_anchor=(0.92, 0, 0.5, 1),
        )

        plt.savefig(name_svg, bbox_inches="tight")

    @staticmethod
    def calc_pct(pct: int, values: list) -> str:
        return "{} min\n{:.2f}%".format(int(pct / 100 * sum(values)), pct)


def graph(dates: list, work: list, relaxation: list) -> None:
    path = os.path.dirname(__file__) + "/"

    # If there are old statistics image files, then delete them
    svg_1 = path + "stat-1.svg"
    if os.path.exists(svg_1):
        os.remove(svg_1)
    svg_2 = path + "stat-2.svg"
    if os.path.exists(svg_2):
        os.remove(svg_2)

    # If statistics for the requested period exists in the database, then
    # display it
    if work:
        if 1 < len(work) <= 31:
            PeriodPomodoros(
                dates=dates,
                work=work,
                relaxation=relaxation,
                name_svg=path + "stat-1.svg",
            )

        CountPomodorosTime(
            work=work, relaxation=relaxation, name_svg=path + "stat-2.svg"
        )

    webbrowser.open("file://" + path + "stats.html")


import os
import time
import sqlite3
import datetime


def create_table_statistic():
    sql = """
    CREATE TABLE IF NOT EXISTS 
    statistic(
        date TEXT,
        work INT,
        relaxation INT
    )
    """
    try:
        cursor.execute(sql)  # We execute the SQL query
    except sqlite3.DatabaseError as err:
        print("Error:", err)


def read_today_entry() -> tuple:
    try:
        sql = f"SELECT work, relaxation FROM statistic WHERE date = '{DATE}'"
        cursor.execute(sql)
    except sqlite3.DatabaseError as err:
        print("Error: ", err)

    work = 0
    relaxation = 0
    rows = cursor.fetchall()
    if rows:
        work = rows[0][0]
        relaxation = rows[0][1]

    return [DATE], [work], [relaxation]


def insert_entry(work: int, relaxation: int) -> None:
    sql = """INSERT INTO statistic(date, work, relaxation) VALUES (?, ?, ?)"""
    params = (DATE, work, relaxation)
    try:
        cursor.execute(sql, params)
    except sqlite3.DatabaseError as err:
        print("Error: ", err)
    else:
        conn.commit()  # Save changes to the database


def update_entry(work: int, relaxation: int) -> None:
    sql = f"""
    UPDATE statistic SET work={work}, relaxation={relaxation} 
    WHERE date='{DATE}'
    """
    try:
        cursor.execute(sql)
    except sqlite3.DatabaseError as err:
        print("Error: ", err)
    else:
        conn.commit()


def save_today_entry(work: int, relaxation: int) -> None:
    """
    In this case, you cannot use:
    'INSERT OR REPLACE INTO User VALUES(?,?,?)'
    """
    # First, check if there is a log entry for today
    _, work_db, relaxation_db = read_today_entry()
    if work_db == [0] and relaxation_db == [0]:
        # If the record does not exist, then add it
        insert_entry(work, relaxation)
    else:
        # Otherwise, update the record by ADDING new data
        update_entry(int(*work_db) + work, int(*relaxation_db) + relaxation)


def read_all_entry() -> tuple:
    try:
        cursor.execute("SELECT date, work, relaxation FROM statistic")
    except sqlite3.DatabaseError as err:
        print("Error: ", err)

    dates = []
    work = []
    relaxation = []
    rows = cursor.fetchall()
    if rows:
        for row in rows:
            dates.append(row[0])
            work.append(row[1])
            relaxation.append(row[2])

    return dates, work, relaxation


def delete_today_entry() -> None:
    try:
        cursor.execute(f"DELETE FROM statistic WHERE date='{DATE}'")
    except sqlite3.DatabaseError as err:
        print("Error: ", err)
    else:
        conn.commit()


def delete_all_entry() -> None:
    try:
        cursor.execute("DELETE FROM statistic")
    except sqlite3.DatabaseError as err:
        print("Error: ", err)
    else:
        conn.commit()


def close_db():
    cursor.close()
    conn.close()


# Create a connection object.
# If the database file is not found in the specified directory, then it will
# be created.
conn = sqlite3.connect(os.path.dirname(__file__) + "/statistics.db")

# According to the DB-API 2.0 specification, after creating a connection
# object, you must create a cursor object. All further requests must be
# made through this object. Creating a cursor object is done using the
# cursor() method.
# Create a cursor object (all database queries are executed through it).
cursor = conn.cursor()

# Today's date
DATE = str(datetime.datetime.fromtimestamp(time.time()).strftime("%Y-%m-%d"))

# Create table statistic if not exists
create_table_statistic()



from pathlib import Path
from setuptools import setup, find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()


def find_requires():
    requirements = []
    dir_path = Path(__file__).resolve().parent
    with open(dir_path / "requirements.txt") as f:
        requirements = f.readlines()

    return requirements


if __name__ == "__main__":
    ver = "0.2.4"

    setup(
        name="pomodorotimer",
        version=ver,
        url="https://github.com/patsuckow/pomodorotimer",
        project_urls={
            "Bug Tracker": "https://github.com/patsuckow/pomodorotimer/issues",
            "Donate": "https://www.paypal.me/patsuckow",
        },
        description="CLI Pomodoro Timer",
        long_description=long_description,
        long_description_content_type="text/markdown",
        license="GNU General Public License v3 (GPLv3)",
        author="Alexey Patsukov 🇷🇺",
        author_email="patsuckow@yandex.ru",
        python_requires=">=3.10.13",
        keywords=[
            "pomodoro",
            "tomato",
            "pomodorotimer",
            "pomodoro timer",
            "pomodoro technique",
            "productivity",
            "time management",
            "countdown",
            "stopwatch",
            "timer",
            "eta",
            "freelance",
            "freelancing",
            "CLI",
        ],
        classifiers=[
            # As in https://pypi.python.org/pypi?:action=list_classifiers
            "Development Status :: 4 - Beta",
            "Environment :: Console",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Intended Audience :: Developers",
            "Topic :: Utilities",
        ],
        # Specify an executable console file
        entry_points={
            "console_scripts": [
                "pomodoro = pomodorotimer.pomodorotimer:PomodoroTimer",
            ],
        },
        include_package_data=True,
        packages=find_packages(
            exclude=[
                # '*.tests', '*.tests.*', 'tests',
                # '*.img', '*.img.*', 'img',
                # '*..github', '*..github.*', '.github'
            ]
        ),
        zip_safe=False,  # the package can run out of an .egg file
        install_requires=find_requires(),
        setup_requires=[],
        # test_suite='tests',
        # tests_require=[],
    )